/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ListProcess;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.UIObject;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Commons.ActionClass;
import org.ow2.orchestra.console.GWTConsole.client.Commons.ActionImage;
import org.ow2.orchestra.console.GWTConsole.client.Commons.IndexFocusPanel;
import org.ow2.orchestra.console.GWTConsole.client.Commons.RowLabel;
import org.ow2.orchestra.console.GWTConsole.client.Commons.SpecialGrid;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.PListInstance;
import org.ow2.orchestra.console.GWTConsole.client.ListProcessActivity.PListProcessActivity;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;
import org.ow2.orchestra.console.GWTConsole.client.Resources.ImageResources;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;


/**
 * Class ProcessGrid that represents a grid containing processes.
 * The class stores the process in its grid and in a list.
 *
 * @author Mathieu Crassin
 *
 */
public class ProcessGrid extends SpecialGrid {

  private List<MProcess> mProcessList;

  private int idLimitSize = 0;
  private int nameLimitSize = 0;
  private int nameSpaceLimitSize = 0;
  private final String gridDebugIdString = "processListGridId";

  protected OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  private PTabLayoutPanel presenterTabLayoutPanel = null;

  protected HandlerManager eventBus = null;

  private static String []columnTitles = {"UUID","Name","Namespace","Version","Deployment Date","Undeployment Date","State","Actions"};


  /**
   * Constructor that init the grid with the column titles.
   *
   * @author Mathieu Crassin
   *
   */
  public ProcessGrid() {
    this(true);
  }

  /**
   * Constructor that init the grid with the column titles.
   * @param isSortActive should sorting be enabled
   */
  public ProcessGrid(final boolean isSortActive) {
    super(ProcessGrid.columnTitles, 6, isSortActive);
  }

  /**
   * Define some rules use to display information in the grid.
   *
   * @author Mathieu Crassin
   * @param idLimitSize : string limit size of the id.
   * @param nameLimitSize
   * @param nameSpaceLimitSize
   * @param exceededString : string to add at the end of too wide strings.
   *
   */
  public void setStringLimitSize(final int idLimitSize, final int nameLimitSize,
      final int nameSpaceLimitSize, final String exceededString) {
    this.idLimitSize = idLimitSize;
    this.nameLimitSize = nameLimitSize;
    this.nameSpaceLimitSize = nameSpaceLimitSize;
    this.exceededString = exceededString;
  }

  /**
   * This method is used to sort and display the processes' list depending on the column and order values
   *
   * @author Jean-Yves Goubet
   *
   */
  @Override
  public void sortAndDisplayList() {
    if (this.mProcessList != null) {
      final MProcessCompare comparator = new MProcessCompare(this.sortColumnIndex, this.order);
      Collections.sort(this.mProcessList, comparator);
      this.manageGrid();
      for (final MProcess m : this.mProcessList) {
        this.addProcess(m);
      }
    }
  }

  /**
   * This method is called to refresh processes' list displayed on the client's screen.
   * For now the method consists in refreshing the whole list each time it's called.
   *
   * @author Jean-Yves Goubet
   *
   */
  @Override
  public void refreshCurrentList() {
    this.orchestramanagement.getProcesses(new AsyncCallback<List<MProcess>>() {
      public void onSuccess(final List<MProcess> mProcessList) {
        ProcessGrid.this.mProcessList = mProcessList;
        ProcessGrid.this.sortAndDisplayList();
        ProcessGrid.this.eventBus.fireEvent(new RefreshDoneEvent());
      }

      public void onFailure(final Throwable caught) {
        GWT.log("ProcessGrid - refreshCurrentList: onFailure : " + caught.getMessage());
        ProcessGrid.this.eventBus.fireEvent(new SuspendTimerEvent(true));
        if (caught instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
              ((OrchestraManagementException)caught).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
  }

  /**
   * This method is used to call the undeployProcess method on orchestra management servlet.
   *
   * @author Jean-Yves Goubet
   * @param rowIndex : the row index of the process
   *
   */
  private void undeployProcess(final int rowIndex) {
    final String processId = this.getText(rowIndex, 0);
    this.orchestramanagement.undeployFromOrchestra(processId,
        new AsyncCallback<Void>() {
          public void onSuccess(final Void arg0) {
            ProcessGrid.this.eventBus.fireEvent(new ViewRefreshEvent(processId, PTab.Type.LIST_PROCESS_ACTIVITY, true));
          }
          public void onFailure(final Throwable caught) {
            GWT.log("PListProcess - undeployProcess: on Failure : " + caught.getMessage());
            if (caught instanceof OrchestraManagementException) {
              PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                  ((OrchestraManagementException)caught).getDetails());
            } else {
              PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while undeploy file !", caught.getMessage());
            }
          }
        });
  }

  /**
   * This method is used to call the retireProcess method on orchestra management servlet.
   *
   * @author Guillaume Porcher
   * @param rowIndex : the row index of the process
   *
   */
  private void retireProcess(final int rowIndex) {
    final String processId = this.getText(rowIndex, 0);
    this.orchestramanagement.retireProcess(processId,
        new AsyncCallback<Void>() {
          public void onSuccess(final Void arg0) {
            ProcessGrid.this.eventBus.fireEvent(new ViewRefreshEvent(processId, PTab.Type.LIST_PROCESS_ACTIVITY, true));
          }
          public void onFailure(final Throwable caught) {
            GWT.log("PListProcess - retireProcess: on Failure : " + caught.getMessage());
            if (caught instanceof OrchestraManagementException) {
              PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                  ((OrchestraManagementException)caught).getDetails());
            } else {
              PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while retiring process !", caught.getMessage());
            }
          }
        });
  }

  /**
   * This method is used to call the activateProcess method on orchestra management servlet.
   *
   * @author Guillaume Porcher
   * @param rowIndex : the row index of the process
   *
   */
  private void activateProcess(final int rowIndex) {
    final String processId = this.getText(rowIndex, 0);
    this.orchestramanagement.activateProcess(processId,
        new AsyncCallback<Void>() {
          public void onSuccess(final Void arg0) {
            ProcessGrid.this.eventBus.fireEvent(new ViewRefreshEvent(processId, PTab.Type.LIST_PROCESS_ACTIVITY, true));
          }
          public void onFailure(final Throwable caught) {
            GWT.log("PListProcess - activateProcess: on Failure : " + caught.getMessage());
            if (caught instanceof OrchestraManagementException) {
              PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                  ((OrchestraManagementException)caught).getDetails());
            } else {
              PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while activating process !", caught.getMessage());
            }
          }
        });
  }

  /**
   * This method is used to call the removeProcess method on orchestra management servlet.
   *
   * @author Jean-Yves Goubet
   * @param rowIndex : the row index of the process
   *
   */
  private void removeProcess(final int rowIndex) {
    final String processId = this.getText(rowIndex, 0);
    this.orchestramanagement.removeProcess(processId,
        new AsyncCallback<Void>() {
          public void onSuccess(final Void arg0) {
            // delete list instance of a process tab if exists
            ProcessGrid.this.presenterTabLayoutPanel.removeTab(processId, PTab.Type.LIST_INSTANCE);
            // delete also activity list tab if exists
            ProcessGrid.this.presenterTabLayoutPanel.removeTab(processId, PTab.Type.LIST_PROCESS_ACTIVITY);
            ProcessGrid.this.eventBus.fireEvent(new ViewRefreshEvent(processId, PTab.Type.LIST_PROCESS_ACTIVITY, true));
          }
          public void onFailure(final Throwable caught) {
            GWT.log("PListProcess - undeployProcess: on Failure : " + caught.getMessage());
            if (caught instanceof OrchestraManagementException) {
              PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                  ((OrchestraManagementException)caught).getDetails());
            } else {
              PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while removing process !", caught.getMessage());
            }
          }
        });
  }

  /**
   * This method is used to add a process in the processes list displayed.
   *
   * @author Jean-Yves Goubet
   * @param process : it's the process to add to the grid
   *
   */
  protected void addProcess(final MProcess process) {
    // add data to the end of the grid
    final int rowIndex = this.getRowCount();
    //instantiate labels
    final RowLabel[] rowLabels = new RowLabel[ProcessGrid.columnTitles.length - 1];
    rowLabels[0] = this.formatLabel(process.getProcessId(), this.idLimitSize, rowIndex);
    rowLabels[1] = this.formatLabel(process.getProcessName(), this.nameLimitSize, rowIndex);
    rowLabels[2] = this.formatLabel(process.getProcessNamespace(), this.nameSpaceLimitSize, rowIndex);
    rowLabels[3] = new RowLabel(Integer.toString(process.getProcessVersion()), rowIndex, false);
    rowLabels[4] = new RowLabel(SpecialGrid.dateFormatter(process.getProcessDeploymentDate()),rowIndex,false);
    rowLabels[5] = (process.getProcessState().toString().equals("undeployed")) ? new RowLabel(SpecialGrid.
        dateFormatter(process.getProcessUndeploymentDate()),rowIndex,false) : new RowLabel("N/A",rowIndex,false);
    rowLabels[6] = new RowLabel(process.getProcessState(),rowIndex,false);
    //addHandler to labels and add labels to grid
    this.addData(rowLabels);
    this.addActionsToProcess(rowIndex, process.getProcessState());

    // define the Debug Id
    this.setDebugId();
  }


  /**
   * This method is used to fill in the actions column depending on process' state.
   *
   * @author Jean-Yves Goubet
   * @param rowIndex : the row index of the process concerned
   * @param state : the state of the process
   *
   */
  private void addActionsToProcess(final int rowIndex, final String state) {
    final ActionImage [] tab = new ActionImage[6];

    // instances action
    tab[0] = new ActionImage(ImageResources.INSTANCE.instancesIcon(),
        ImageResources.INSTANCE.instancesIconDisabled(), rowIndex);
    tab[0].setTitle("instances");

    // activities action
    tab[1] = new ActionImage(ImageResources.INSTANCE.activities(),
        ImageResources.INSTANCE.activitiesDisabled(), rowIndex);
    tab[1].setTitle("activities");

    // retire action
    tab[2] = new ActionImage(ImageResources.INSTANCE.suspend(),
        ImageResources.INSTANCE.suspendDisabled(), rowIndex);
    tab[2].setTitle("retire");
    // activate action
    tab[3] = new ActionImage(ImageResources.INSTANCE.resume(),
        ImageResources.INSTANCE.resumeDisabled(), rowIndex);
    tab[3].setTitle("activate");

    // undeploy action
    tab[4] = new ActionImage(ImageResources.INSTANCE.undeployIcon(),
        ImageResources.INSTANCE.undeployIconDisabled(), rowIndex);
    tab[4].setTitle("undeploy");

    // delete action
    tab[5] = new ActionImage(ImageResources.INSTANCE.deleteIcon(),
        ImageResources.INSTANCE.deleteIconDisabled(), rowIndex);
    tab[5].setTitle("remove");

    this.addActions(tab, state, 7);
  }

  @Override
  protected void addClickHandlers(final ActionImage[] tabImages, final String state) {

    // instances list
    tabImages[0].setHandler(new ActionClass() {
      public void execute() {
        // create list instance presenter
        new PListInstance(ProcessGrid.this.presenterTabLayoutPanel,      // also give the tabLayoutPanel (thus the presenter could manage tab)
            ProcessGrid.this.getText(tabImages[0].getRowIndex(), 0), // and to finish, give the process Id
            ProcessGrid.this.getText(tabImages[0].getRowIndex(), 1),
            ProcessGrid.this.eventBus
        );
      }
    });


    // activities
    tabImages[1].setHandler(new ActionClass() {
      public void execute() {
        final String processId = ProcessGrid.this.getText(tabImages[1].getRowIndex(),0);
        final String processName = ProcessGrid.this.getText(tabImages[1].getRowIndex(), 1);

        // create the presenter of list process activity
        new PListProcessActivity(ProcessGrid.this.presenterTabLayoutPanel, processId, processName, ProcessGrid.this.eventBus);
      }
    });

    // undeployed
    if (state.equals("undeployed")) {
      tabImages[2].setEnabled(false);
      tabImages[3].setEnabled(false);
      tabImages[4].setEnabled(false);
      tabImages[5].setEnabled(true);

      tabImages[5].setHandler(new ActionClass() {
        public void execute() {
          ProcessGrid.this.removeProcess(tabImages[5].getRowIndex());
        }
      });
    }
    // deployed
    else if (state.equals("active")) {
      tabImages[2].setEnabled(true);
      tabImages[3].setEnabled(false);
      tabImages[4].setEnabled(true);
      tabImages[5].setEnabled(false);

      tabImages[2].setHandler(new ActionClass() {
        public void execute() {
          ProcessGrid.this.retireProcess(tabImages[2].getRowIndex());
        }
      });
      tabImages[4].setHandler(new ActionClass() {
        public void execute() {
          ProcessGrid.this.undeployProcess(tabImages[4].getRowIndex());
        }
      });
    } else if (state.equals("retired")) {
      tabImages[2].setEnabled(false);
      tabImages[3].setEnabled(true);
      tabImages[4].setEnabled(true);
      tabImages[5].setEnabled(false);

      tabImages[3].setHandler(new ActionClass() {
        public void execute() {
          ProcessGrid.this.activateProcess(tabImages[3].getRowIndex());
        }
      });
      tabImages[4].setHandler(new ActionClass() {
        public void execute() {
          ProcessGrid.this.undeployProcess(tabImages[4].getRowIndex());
        }
      });
    }
  }

  /**
   * This class is the comparator used to sort the list of processes
   *
   * @author Jean-Yves Goubet
   *
   */
  private class MProcessCompare implements Comparator<MProcess> {
    private final int compareType;
    private final boolean order;

    public int compare(final MProcess o1, final MProcess o2) {
      switch (this.compareType) {
        //UUID
        case 0:
          return this.order ? Integer.valueOf(o1.getProcessId()).compareTo(Integer.valueOf(o2.getProcessId()))
            : Integer.valueOf(o2.getProcessId()).compareTo(Integer.valueOf(o1.getProcessId()));
        //Name
        case 1:
          return this.order ? o1.getProcessName().compareTo(o2.getProcessName())
            : o2.getProcessName().compareTo(o1.getProcessName());
        //Namespace
        case 2:
          return this.order ? o1.getProcessNamespace().compareTo(o2.getProcessNamespace())
            : o2.getProcessNamespace().compareTo(o1.getProcessNamespace());
        //Deployment Date
        case 3:
          return this.order ? o1.getProcessDeploymentDate().compareTo(o2.getProcessDeploymentDate())
            : o2.getProcessDeploymentDate().compareTo(o1.getProcessDeploymentDate());
        //Undeployment Date
        case 4:
          if (o1.getProcessState().equals("deployed") && o1.getProcessState().equals(o2.getProcessState())) {
            return 0;
          } else if (o1.getProcessState().equals("deployed")) {
            return this.order ? -1 : 1;
          } else if (o2.getProcessState().equals("deployed")) {
            return this.order ? 1 : -1;
          } else {
            return this.order ? o1.getProcessUndeploymentDate().compareTo(o2.getProcessUndeploymentDate())
                : o2.getProcessUndeploymentDate().compareTo(o1.getProcessUndeploymentDate());
          }
        //State
        case 5:
          return this.order ? o1.getProcessState().compareTo(o2.getProcessState())
            : o2.getProcessState().compareTo(o1.getProcessState());
        //default case (return 0 as if the 2 elements to compare were equals)
        default:
          return 0;
      }
    }

    public MProcessCompare(final int compareType, final boolean order) {
      this.compareType = compareType;
      this.order = order;
    }
  }

  /**
   * Define the debug id name of the grid.
   *
   * @author Mathieu Crassin
   */
  protected void setDebugId() {
    this.ensureDebugId(this.gridDebugIdString);
  }

  /**
   * Define a particular behavior for the function ensureDebugId.
   * Thus we can name the grid, the cell of the grid, the icons in actions column.
   *
   * @author Mathieu Crassin
   */
  @Override
  protected void onEnsureDebugId(final String baseID) {
    super.onEnsureDebugId(baseID);

    final int row = this.getRowCount();
    final int col = this.getColumnCount();
    if (row > 1 && col == 8) {

      for (int i = 1; i < row; i++) {
        UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(i, 7)).getWidget()).
            getWidget(0).getElement(), baseID + "-" + i + "-" + (col - 1), "instancesImageID");
        UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(i, 7)).getWidget()).
            getWidget(1).getElement(), baseID + "-" + i + "-" + (col - 1), "activitiesImageID");
        UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(i, 7)).getWidget()).
            getWidget(2).getElement(), baseID + "-" + i + "-" + (col - 1), "retireImageID");
        UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(i, 7)).getWidget()).
            getWidget(3).getElement(), baseID + "-" + i + "-" + (col - 1), "activateImageID");
        UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(i, 7)).getWidget()).
            getWidget(4).getElement(), baseID + "-" + i + "-" + (col - 1), "undeployImageID");
        UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(i, 7)).getWidget()).
            getWidget(5).getElement(), baseID + "-" + i + "-" + (col - 1), "removeImageID");
      }
    }
  }

  /**
   * Allow the process grid class to access to the tab organization.
   *
   * @param tabLayoutPanel
   */
  public void setPresenterTabLayoutPanel(final PTabLayoutPanel presenterTabLayoutPanel) {
    this.presenterTabLayoutPanel  = presenterTabLayoutPanel;
  }


  /**
   * register the eventBus in ProcessGrid class
   *
   * @author Jean-Yves Goubet
   * @param eventBus : the handler manager of the application
   */
  public void setHandlerManager(final HandlerManager eventBus) {
    this.eventBus  = eventBus;
  }
}

